<?php

namespace Classes;

/**
 * Loader
 * 
 * Load common things, such as controllers, sections, etc...
 */
class Load {
	
	/**
	 * Loads a skeleton class
	 * 
	 * Skeleton-classes are used for either extending or constructing.
	 *
	 * @param string $name
	 */
	private function _load_skeleton($name) {
		require_once SYSTEM.'/skeletons/'.$name.'.php';
	}
	
	/**
	 * Load an instance of a controller-class
	 *
	 * @param string $name
	 *
	 * @return Controller
	 */
	public function controller($name) {
		
		$this->_load_skeleton('controller');
		
		if(is_file($path = SYSTEM.'/controllers/'.strtolower($name).'.php')) {
			
			// Next load the actual class
			require_once $path;
			
			//Then create and return an instance of it
			$real_class_name = 'Controllers\\'.$name;
			return new $real_class_name;
			
		}
		
		error('Controller `'.$name.'` does not exist.');
		
	}
	
	/**
	 * Loads an instance of a view
	 *
	 * @param string $name
	 * @return View
	 */
	public function view($name) {
		
		$this->_load_skeleton('view');
		
		if(is_file($path = SYSTEM.'/views/'.strtolower($name).'.php')) {
			
			// Create a new View based on skeleton
			$view = new \Skeletons\View;
			
			// Give it the path
			$view->path = $path;
			
			// Return it
			return $view;
			
		}
		
		error('View `'.$name.'` does not exist.');
		
	}
	
	/**
	 * Loads an instance of a section
	 *
	 * @param string $name
	 * @return Section
	 */
	public function section($name) {
		
		$this->_load_skeleton('section');
		
		if(is_file($path = APP.'/sections/'.$name.'.php')) {
			
			// Create a new Section based on skeleton
			$section = new \Skeletons\Section;
			
			// Give it the configuration
			$section->load_configuration($name, require $path);
			
			// Return it
			return $section;
			
		}
		
		error('Section `'.$name.'` does not exist.');
		
	}
	
	/**
	 * Loads an instance of a type
	 *
	 * @param string $name
	 * @return Type
	 */
	public function type($name) {
		
		$this->_load_skeleton('type');
		
		if(is_file($path = APP.'/types/'.$name.'.php')) {
			
			// Next load the actual class
			require_once $path;
			
			//Then create and return an instance of it
			$real_class_name = 'Types\\'.$name;
			return new $real_class_name;
			
		}
		
		// We got this far, it's an error, so let's
		// return the default type
		return $this->type(Config::get('general', 'default_type'));
		
	}
	
	/**
	 * Loads an instance of a auth-driver
	 *
	 * @param string $name
	 * @return AuthDriver
	 */
	public function auth_driver($name) {
		
		$this->_load_skeleton('auth');
		
		if(is_file($path = APP.'/authdrivers/'.$name.'.php')) {
			
			// Next load the actual class
			require_once $path;
			
			// Then create and return an instance of it
			$real_class_name = 'AuthDrivers\\'.ucwords($name);
			return new $real_class_name;
			
		}
		
		error('Auth-driver `'.$name.'` does not exist.');
		
	}
	
	/**
	 * Loads the contents of a module
	 *
	 * @param string $name
	 * @return string
	 */
	public function module($name) {
		
		if(is_file($path = APP.'/modules/'.$name.'.php')) {
			
			ob_start();
			require $path;
			return ob_get_clean();
			
		}
		
		error('View `'.$name.'` does not exist.');
		
	}
	
	/**
	 * Load a language-file
	 *
	 * @param string $name
	 * @return array
	 */
	public function language($name) {
		
		if(is_file($path = APP.'/languages/'.$name.'.php')) {
			return require $path;
		}
		
		error('Language `'.$name.'` does not exist.');
		
	}
	
}